<?php

use Symfony\Component\HttpFoundation\JsonResponse;

function themler_output($response) {
    return new JsonResponse($response);
}

function themler_update() {
    $theme_name = $_POST['themeName'];
    $preview_theme_name = get_preview_theme_name($theme_name);
    $response = update($theme_name);
    return themler_output($response);
}

function themler_export() {
    $theme_name = $_POST['themeName'];
    $info = get_export_info();
    $response = export($theme_name, $info);
    return themler_output($response);
}

function themler_save() {
    $theme_name = $_POST['themeName'];
    $info = get_export_info();
    $isNew = false;
    $response = save($info, $theme_name, $isNew);
    return themler_output($response);
}

function themler_download() {
    $theme_name = isset($_REQUEST['id']) ? $_REQUEST['id'] : // download by id
                  (isset($_REQUEST['srcTheme']) ? $_REQUEST['srcTheme'] : $_REQUEST['themeName']); // download by url

    $user_theme_name = isset($_REQUEST['themeName']) ? $_REQUEST['themeName'] : '';
    $include_editor = !isset($_REQUEST['includeEditor']) || $_REQUEST['includeEditor'] === 'true';
    $archive = download($theme_name, $user_theme_name, $include_editor);

    header('Content-Type: application/zip');
    header('Content-Disposition: inline; filename="'.$archive['name'].'"');
    header('Pragma: no-cache');
    echo $archive['content'];
    die;
}

function themler_remove() {
    $theme_name = isset($_POST['removedName']) ? $_POST['removedName'] : '';
    $response = remove_theme($theme_name);
    return themler_output($response);
}

function themler_copy() {
    $theme_name = $_POST['themeName'];
    $new_theme_name = isset($_POST['newName']) ? $_POST['newName'] : '';
    $response = copy_theme($theme_name, $new_theme_name);
    return themler_output($response);
}

function themler_rename() {
    $theme_name = $_POST['themeName'];
    $new_theme_name = isset($_POST['newName']) ? $_POST['newName'] : '';
    $response = rename_theme($theme_name, $new_theme_name);
    return themler_output($response);
}

function themler_canrename() {
    $theme_name = $_POST['themeName'];
    $new_theme_name = isset($_POST['newName']) ? $_POST['newName'] : '';
    $response = can_rename($theme_name, $new_theme_name);
    return themler_output($response);
}

function themler_upload($params) {
    $callback = $params['callback'];
    $theme_name = $params['theme'];
    $response = upload($theme_name, $callback);
    return themler_output($response);
}


function themler_validation() {
    $new_theme_name = isset($_POST['newName']) ? $_POST['newName'] : '';
    $response = theme_name_validate($new_theme_name);
    return themler_output($response);
}

function themler_get_files() {
    $theme_name = $_POST['themeName'];
    $mask = isset($_POST['mask']) ? $_POST['mask'] : null;
    $filter = isset($_POST['filter']) ? $_POST['filter'] : null;
    $response = get_files($theme_name, $mask, $filter);
    return themler_output($response);
}

function themler_set_files() {
    $response = set_files();
    return themler_output($response);
}

function themler_fso_zip() {
    $theme_name = $_POST['themeName'];
    $response = fso_zip($theme_name);
    return themler_output($response);
}

function themler_clear() {
    $response = clear_chunk();
    return themler_output($response);
}

function themler_activate() {
    $theme_name = $_POST['themeName'];
    $new_theme_name = isset($_POST['newName']) ? $_POST['newName'] : '';
    $response = activate_theme($theme_name, $new_theme_name);
    return themler_output($response);
}

function themler_refresh_templates() {
    $theme_name = $_POST['themeName'];
    $base_url = get_base_url_with_slash();
    $response = array('templates' => get_templates_url($base_url, $theme_name));
    return themler_output($response);
}

function themler_refresh_themes() {
    $theme_name = $_POST['themeName'];
    $response = get_themes_info();
    return themler_output($response);
}

function user_is_logged_in() {
    $account = \Drupal::currentUser();
    return $account && $account->isAuthenticated();
}

function themler_login_user() {
    ProviderLog::start('themler_login_user');
    theme_check_memory_limit();
    if (isset($_REQUEST['uid']) && !user_is_logged_in()) {
        $account = \Drupal::entityManager()->getStorage('user')->load($_REQUEST['uid']);
        user_login_finalize($account);
    }
    ProviderLog::end('themler_login_user');
}

function get_export_info() {
    $data = isset($_POST['info']) ? $_POST['info'] : false;
    if ($data === false) {
        $info = getChunkInfo();
    } else {
        $info = json_decode($data, true);
    }
    if (null === $info) {
        trigger_error('Data is incorrect: "' . $data . '"', E_USER_ERROR);
    }
    return $info;
}

function themler_show($theme_name) {
    $base_url = get_base_url_with_slash();
    $urls = get_urls($base_url, $theme_name);
    $process_request = $urls['process_request'];
    $theme_dir = get_theme_dir($theme_name);
    $themler_url = base_path().drupal_get_path('module', 'themler') . '/js';
    $manifestVersion = theme_get_manifest_version($theme_name);
    theme_copy_manifest($theme_name, $manifestVersion);
    $version = time();

    $output = <<<EOT
<!DOCTYPE html>
<html manifest="$process_request&action=init_manifest&ver=$manifestVersion">
<head>
<title>Themler</title>
<meta http-equiv="X-UA-Compatible" content="IE=edge" />

<script type="text/javascript" src="$process_request&action=init_project&themeName=$theme_name&version=$version"></script>
<script type="text/javascript" src="$themler_url/DataProvider.js?version=$version"></script>
<script type="text/javascript" src="$themler_url/loader.js?version=$version"></script>
</head>
<body>
</body>
</html>
EOT;
    return $output;
}

function themler_init_project($params) {
    if (!user_is_logged_in())
        die('User is not logged in.');

    $account = \Drupal::currentUser();
    $theme_name = $params['themeName'];
    $theme_dir = get_theme_dir($theme_name);
    $base_url = get_base_url_with_slash();
    $templates = get_templates_url($base_url, $theme_name);
    $isNew = isset($params['create_theme']) ? $params['create_theme'] : 'false'; // for save and publish new theme
    $isActive = 'false';

    $data = json_encode(array(
        'uid' => $account->id(),
        'themeName' => $theme_name,
        'templates' => $templates,
        'canDuplicateTemplatesConstructors' => get_duplicated_templates(),
        'themes' => get_themes_info(),
        'url' => get_urls($base_url, $theme_name),
        'isActive' => $isActive,
        'isNew' => $isNew,
        'cmsVersion' => get_versions(),
        'cssJsSources' => get_theme_cache($theme_dir),
        'md5Hashes' => get_theme_hashes($theme_dir),
        'project' => get_project($theme_dir),
        'maxRequestSize' => get_max_request_size()
    ));

    header('Content-Type: application/javascript');
    header("Pragma: no-cache");
    echo("window.pluginData = $data;");
    die;
}

function themler_init_manifest($params) {
    if (!user_is_logged_in())
        die('User is not logged in.');

    header('Content-Type: text/cache-manifest');
    header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
    header("Cache-Control: post-check=0, pre-check=0", false);
    header("Pragma: no-cache");

    echo(theme_load_manifest($params['ver']));
    die;
}